@@ -9,70 +9,34 @@ use Carp qw();
 require Exporter;
 our @ISA = qw(Exporter);
 
-our $VERSION = '0.7';
+our $VERSION = '1.100470';
 
 our @EXPORT    = qw(LookUp LoadDB);
 our @EXPORT_OK = @EXPORT;
 
-my $def_db = 'ipscountry.dat';
-my @baseX  = (
+my $DEFAULT_DB   = 'ipscountry.dat';
+my $cache_expire = 1000;
+my @baseX        = (
     0 .. 9,
     'A' .. 'Z',
     'a' .. 'z',
     split( m{}, q(.,;'"`<>{}[]=+-~*@#%$&!?) )
 );
 
-my %countrys = (
-    qw(
-        -- N/A L0 localhost I0 IntraNet A1 Anonymous_Proxy A2 Satellite_Provider
-        AD Andorra AE United_Arab_Emirates AF Afghanistan AG Antigua_and_Barbuda AI Anguilla AL Albania AM Armenia AN Netherlands_Antilles
-        AO Angola AP Asia/Pacific_Region AQ Antarctica AR Argentina AS American_Samoa AT Austria AU Australia AW Aruba AZ Azerbaijan BA Bosnia_and_Herzegovina BB Barbados BD Bangladesh
-        BE Belgium BF Burkina_Faso BG Bulgaria BH Bahrain BI Burundi BJ Benin BM Bermuda BN Brunei_Darussalam BO Bolivia BR Brazil BS Bahamas BT Bhutan BV Bouvet_Island BW Botswana
-        BY Belarus BZ Belize CA Canada CC Cocos_(Keeling)_Islands CD),
-    'Congo,_The_Democratic_Republic_of_the',
-    qw(CF Central_African_Republic CG Congo CH Switzerland CI Cote_D'Ivoire CK Cook_Islands
-        CL Chile CM Cameroon CN China CO Colombia CR Costa_Rica CU Cuba CV Cape_Verde CX Christmas_Island CY Cyprus CZ Czech_Republic DE Germany DJ Djibouti DK Denmark DM Dominica
-        DO Dominican_Republic DZ Algeria EC Ecuador EE Estonia EG Egypt EH Western_Sahara ER Eritrea ES Spain ET Ethiopia EU Europe FI Finland FJ Fiji FK Falkland_Islands_(Malvinas)
-        FM), 'Micronesia,_Federated_States_of',
-    qw(FO Faroe_Islands FR France FX), 'France,_Metropolitan',
-    qw(GA Gabon GB United_Kingdom GD Grenada GE Georgia GF French_Guiana GH Ghana GI Gibraltar
-        GL Greenland GM Gambia GN Guinea GP Guadeloupe GQ Equatorial_Guinea GR Greece GS South_Georgia_and_the_South_Sandwich_Islands GT Guatemala GU Guam GW Guinea-Bissau GY Guyana
-        HK Hong_Kong HM Heard_Island_and_McDonald_Islands HN Honduras HR Croatia HT Haiti HU Hungary ID Indonesia IE Ireland IL Israel IN India IO British_Indian_Ocean_Territory
-        IQ Iraq IR), 'Iran,_Islamic_Republic_of',
-    qw(IS Iceland IT Italy JM Jamaica JO Jordan JP Japan KE Kenya KG Kyrgyzstan KH Cambodia KI Kiribati KM Comoros KN Saint_Kitts_and_Nevis
-        KP), "Korea,_Democratic_People's_Republic_of", 'KR',
-    'Korea,_Republic_of',
-    qw(KW Kuwait KY Cayman_Islands KZ Kazakhstan LA Lao_People's_Democratic_Republic LB Lebanon LC Saint_Lucia LI Liechtenstein
-        LK Sri_Lanka LR Liberia LS Lesotho LT Lithuania LU Luxembourg LV Latvia LY Libyan_Arab_Jamahiriya MA Morocco MC Monaco MD),
-    'Moldova,_Republic_of', qw(MG Madagascar MH Marshall_Islands
-        MK), 'Macedonia,_the_Former_Yugoslav_Republic_of',
-    qw(ML Mali MM Myanmar MN Mongolia MO Macau MP Northern_Mariana_Islands MQ Martinique MR Mauritania MS Montserrat MT Malta MU Mauritius
-        MV Maldives MW Malawi MX Mexico MY Malaysia MZ Mozambique NA Namibia NC New_Caledonia NE Niger NF Norfolk_Island NG Nigeria NI Nicaragua NL Netherlands NO Norway NP Nepal
-        NR Nauru NU Niue NZ New_Zealand OM Oman PA Panama PE Peru PF French_Polynesia PG Papua_New_Guinea PH Philippines PK Pakistan PL Poland PM Saint_Pierre_and_Miquelon PN Pitcairn
-        PR Puerto_Rico PS), 'Palestinian_Territory,_Occupied',
-    qw(PT Portugal PW Palau PY Paraguay QA Qatar RE Reunion RO Romania RU Russian_Federation RW Rwanda SA Saudi_Arabia SB Solomon_Islands
-        SC Seychelles SD Sudan SE Sweden SG Singapore SH Saint_Helena SI Slovenia SJ Svalbard_and_Jan_Mayen SK Slovakia SL Sierra_Leone SM San_Marino SN Senegal SO Somalia SR Suriname
-        ST Sao_Tome_and_Principe SV El_Salvador SY Syrian_Arab_Republic SZ Swaziland TC Turks_and_Caicos_Islands TD Chad TF French_Southern_Territories TG Togo TH Thailand TJ Tajikistan
-        TK Tokelau TM Turkmenistan TN Tunisia TO Tonga TP East_Timor TR Turkey TT Trinidad_and_Tobago TV Tuvalu TW),
-    'Taiwan,_Province_of_China', 'TZ', 'Tanzania,_United_Republic_of',
-    qw(UA Ukraine
-        UG Uganda UM United_States_Minor_Outlying_Islands US United_States UY Uruguay UZ Uzbekistan VA Holy_See_(Vatican_City_State) VC Saint_Vincent_and_the_Grenadines VE Venezuela
-        VG), 'Virgin_Islands,_British', 'VI', 'Virgin_Islands,_U.S.',
-    qw(VN Vietnam VU Vanuatu WF Wallis_and_Futuna WS Samoa YE Yemen YT Mayotte YU Yugoslavia ZA South_Africa ZM Zambia ZR Zaire ZW Zimbabwe
-        )
-);
+my ( %baseX, $base, $THIS, %countrys );
 
-my ( %baseX, $base, $THIS );
-
-my $cache_expire = 1000;
-
-# DECLARE BASE LIB
 {
     my $c = 0;
     %baseX = map { $_ => ( $c++ ) } @baseX;
     $base = @baseX;
 
-    foreach my $Key ( keys %countrys ) { $countrys{ $Key } =~ s/_/ /gs; }
+    my @data;
+    while ( <DATA> ) {
+        last if m{^__END__};
+        chomp;
+        push @data, split m{ }, $_, 2;
+    }
+    %countrys = @data;
 }
 
 sub new {
@@ -83,10 +47,9 @@ sub new {
         $db_file = $_[ 0 ];
     }
 
-    my $this = {};
-    bless( $this, $class );
+    my $this = bless( {}, $class );
 
-    if ( !defined $db_file ) { $db_file = &find_db_file; }
+    if ( !defined $db_file ) { $db_file = _find_db_file(); }
 
     $this->{ dbfile } = $db_file;
 
@@ -94,14 +57,29 @@ sub new {
 
     $this->{ cache } = 1;
 
-    return ( $this );
+    return $this;
+}
+
+sub _find_db_file {
+    my @locations = (
+        qw(/usr/local/share /usr/local/share/GeoIPfree),
+        map { $_, "$_/Geo" } @INC
+    );
+
+    # lastly, find where this module was loaded, and try that dir
+    my ( $lib ) = ( $INC{ 'Geo/IPfree.pm' } =~ /^(.*?)[\\\/]+[^\\\/]+$/gs );
+    push @locations, $lib;
+
+    for my $file ( map { "$_/$DEFAULT_DB" } @locations ) {
+        return $file if -e $file;
+    }
 }
 
 sub LoadDB {
     my $this = shift;
     my ( $db_file ) = @_;
 
-    if ( -d $db_file ) { $db_file .= "/$def_db"; }
+    if ( -d $db_file ) { $db_file .= "/$DEFAULT_DB"; }
 
     if ( !-s $db_file ) {
         Carp::croak( "Can't load database, blank or not there: $db_file" );
@@ -235,166 +213,416 @@ sub Faster {
 sub Clean_Cache { delete $_[ 0 ]->{ CACHE }; 1; }
 
 sub nslookup {
-    my ( $host ) = @_;
+    my ( $host, $last_lookup ) = @_;
     require Socket;
-    my $iaddr = Socket::inet_aton( $host );
-    $iaddr = '' if !defined $iaddr;
+    my $iaddr = Socket::inet_aton( $host ) || '';
     my @ip = unpack( 'C4', $iaddr );
-    if ( !@ip && !$_[ 1 ] ) { return ( &nslookup( "www.$host", 1 ) ); }
-    return ( join( ".", @ip ) );
-}
 
-sub find_db_file {
-    my @locations = (
-        qw(/usr/local/share /usr/local/share/GeoIPfree),
-        map { $_, "$_/Geo" } @INC
-    );
-
-    # lastly, find where this module was loaded, and try that dir
-    my ( $lib ) = ( $INC{ 'Geo/IPfree.pm' } =~ /^(.*?)[\\\/]+[^\\\/]+$/gs );
-    push @locations, $lib;
-
-    for my $file ( map { "$_/$def_db" } @locations ) {
-        return $file if -e $file;
-    }
+    return nslookup( "www.${host}", 1 ) if !@ip && !$last_lookup;
+    return join( '.', @ip );
 }
 
 sub ip2nb {
     my @ip = split( /\./, $_[ 0 ] );
-    return ( ( $ip[ 0 ] << 24 ) 
-        + ( $ip[ 1 ] << 16 ) 
-            + ( $ip[ 2 ] << 8 )
-            + $ip[ 3 ] );
+    return ( $ip[ 0 ] << 24 ) + ( $ip[ 1 ] << 16 ) + ( $ip[ 2 ] << 8 )
+        + $ip[ 3 ];
 }
 
 sub nb2ip {
-    my ( $ipn ) = @_;
-
+    my ( $input ) = @_;
     my @ip;
 
-    my $x = $ipn;
-
-    while ( $x > 1 ) {
-        my $c  = $x / 256;
-        my $ci = int( $x / 256 );
-
-        #push(@ip, $x - ($ci*256));
-        push( @ip, $x - ( $ci << 8 ) );
-        $x = $ci;
+    while ( $input > 1 ) {
+        my $int = int( $input / 256 );
+        push @ip, $input - ( $int << 8 );
+        $input = $int;
     }
 
-    push( @ip, $x ) if $x > 0;
+    push @ip, $input if $input > 0;
+    push @ip, ( 0 ) x ( 4 - @ip );
 
-    while ( $#ip < 3 ) { push( @ip, 0 ); }
-
-    @ip = reverse( @ip );
-
-    return ( join( ".", @ip ) );
+    return join( '.', reverse @ip );
 }
 
 sub dec2baseX {
     my ( $dec ) = @_;
-
     my @base;
-    my $x = $dec;
 
-    while ( $x > 1 ) {
-        my $c  = $x / $base;
-        my $ci = int( $x / $base );
-        push( @base, $x - ( $ci * $base ) );
-        $x = $ci;
+    while ( $dec > 1 ) {
+        my $int = int( $dec / $base );
+        push @base, $dec - $int * $base;
+        $dec = $int;
     }
 
-    push( @base, $x ) if $x > 0;
+    push @base, $dec if $dec > 0;
+    push @base, ( 0 ) x ( 5 - @base );
 
-    while ( $#base < 4 ) { push( @base, 0 ); }
-
-    my $baseX;
-
-    foreach my $base_i ( reverse @base ) {
-        $baseX .= $baseX[ $base_i ];
-    }
-
-    return ( $baseX );
+    return join( '', map { $baseX[ $_ ] } reverse @base );
 }
 
 sub baseX2dec {
-    my ( $baseX ) = @_;
+    my ( $input ) = @_;
 
-    my @base = split( "", $baseX );
-    my $dec;
+    my @digits = reverse split( '', $input );
+    my $dec = 0;
 
-    my $i = -1;
-    foreach my $base_i ( reverse @base ) {
-        $i++;
-        $dec += $baseX{ $base_i } * ( $base**$i );
+    foreach ( 0 .. @digits - 1 ) {
+        $dec += $baseX{ $digits[ $_ ] } * ( $base**$_ );
     }
 
-    return ( $dec );
+    return $dec;
 }
 
 1;
 
+__DATA__
+-- N/A
+L0 localhost
+I0 IntraNet
+A1 Anonymous Proxy
+A2 Satellite Provider
+AD Andorra
+AE United Arab Emirates
+AF Afghanistan
+AG Antigua and Barbuda
+AI Anguilla
+AL Albania
+AM Armenia
+AN Netherlands Antilles
+AO Angola
+AP Asia/Pacific Region
+AQ Antarctica
+AR Argentina
+AS American Samoa
+AT Austria
+AU Australia
+AW Aruba
+AX Aland Islands
+AZ Azerbaijan
+BA Bosnia and Herzegovina
+BB Barbados
+BD Bangladesh
+BE Belgium
+BF Burkina Faso
+BG Bulgaria
+BH Bahrain
+BI Burundi
+BJ Benin
+BM Bermuda
+BN Brunei Darussalam
+BO Bolivia
+BR Brazil
+BS Bahamas
+BT Bhutan
+BV Bouvet Island
+BW Botswana
+BY Belarus
+BZ Belize
+CA Canada
+CC Cocos (Keeling) Islands
+CD Congo, The Democratic Republic of the
+CF Central African Republic
+CG Congo
+CH Switzerland
+CI Cote D'Ivoire
+CK Cook Islands
+CL Chile
+CM Cameroon
+CN China
+CO Colombia
+CR Costa Rica
+CS Serbia and Montenegro
+CU Cuba
+CV Cape Verde
+CX Christmas Island
+CY Cyprus
+CZ Czech Republic
+DE Germany
+DJ Djibouti
+DK Denmark
+DM Dominica
+DO Dominican Republic
+DZ Algeria
+EC Ecuador
+EE Estonia
+EG Egypt
+EH Western Sahara
+ER Eritrea
+ES Spain
+ET Ethiopia
+EU Europe
+FI Finland
+FJ Fiji
+FK Falkland Islands (Malvinas)
+FM Micronesia, Federated States of
+FO Faroe Islands
+FR France
+FX France Metropolitan
+GA Gabon
+GB United Kingdom
+GD Grenada
+GE Georgia
+GF French Guiana
+GG Guernsey
+GH Ghana
+GI Gibraltar
+GL Greenland
+GM Gambia
+GN Guinea
+GP Guadeloupe
+GQ Equatorial Guinea
+GR Greece
+GS South Georgia and the South Sandwich Islands
+GT Guatemala
+GU Guam
+GW Guinea-Bissau
+GY Guyana
+HK Hong Kong
+HM Heard Island and McDonald Islands
+HN Honduras
+HR Croatia
+HT Haiti
+HU Hungary
+ID Indonesia
+IE Ireland
+IL Israel
+IM Isle of Man
+IN India
+IO British Indian Ocean Territory
+IQ Iraq
+IR Iran, Islamic Republic of
+IS Iceland
+IT Italy
+JE Jersey
+JM Jamaica
+JO Jordan
+JP Japan
+KE Kenya
+KG Kyrgyzstan
+KH Cambodia
+KI Kiribati
+KM Comoros
+KN Saint Kitts and Nevis
+KP Korea, Democratic People's Republic of
+KR Korea, Republic of
+KW Kuwait
+KY Cayman Islands
+KZ Kazakhstan
+LA Lao People's Democratic Republic
+LB Lebanon
+LC Saint Lucia
+LI Liechtenstein
+LK Sri Lanka
+LR Liberia
+LS Lesotho
+LT Lithuania
+LU Luxembourg
+LV Latvia
+LY Libyan Arab Jamahiriya
+MA Morocco
+MC Monaco
+MD Moldova, Republic of
+ME Montenegro
+MF Saint Martin
+MG Madagascar
+MH Marshall Islands
+MK Macedonia, the Former Yugoslav Republic of
+ML Mali
+MM Myanmar
+MN Mongolia
+MO Macau
+MP Northern Mariana Islands
+MQ Martinique
+MR Mauritania
+MS Montserrat
+MT Malta
+MU Mauritius
+MV Maldives
+MW Malawi
+MX Mexico
+MY Malaysia
+MZ Mozambique
+NA Namibia
+NC New Caledonia
+NE Niger
+NF Norfolk Island
+NG Nigeria
+NI Nicaragua
+NL Netherlands
+NO Norway
+NP Nepal
+NR Nauru
+NU Niue
+NZ New Zealand
+OM Oman
+PA Panama
+PE Peru
+PF French Polynesia
+PG Papua New Guinea
+PH Philippines
+PK Pakistan
+PL Poland
+PM Saint Pierre and Miquelon
+PN Pitcairn
+PR Puerto Rico
+PS Palestinian Territory, Occupied
+PT Portugal
+PW Palau
+PY Paraguay
+QA Qatar
+RE Reunion
+RO Romania
+RS Serbia
+RU Russian Federation
+RW Rwanda
+SA Saudi Arabia
+SB SolomonIslands
+SC Seychelles
+SD Sudan
+SE Sweden
+SG Singapore
+SH Saint Helena
+SI Slovenia
+SJ Svalbard and Jan Mayen
+SK Slovakia
+SL Sierra Leone
+SM San Marino
+SN Senegal
+SO Somalia
+SR Suriname
+ST Sao Tome and Principe
+SV El Salvador
+SY Syrian Arab Republic
+SZ Swaziland
+TC Turks and Caicos Islands
+TD Chad
+TF French Southern Territories
+TG Togo
+TH Thailand
+TJ Tajikistan
+TK Tokelau
+TL Timor-Leste
+TM Turkmenistan
+TN Tunisia
+TO Tonga
+TP East Timor
+TR Turkey
+TT Trinidad and Tobago
+TV Tuvalu
+TW Taiwan, Province of China
+TZ Tanzania, United Republic of
+UA Ukraine
+UG Uganda
+UK United Kingdom
+UM United States Minor Outlying Islands
+US United States
+UY Uruguay
+UZ Uzbekistan
+VA Holy See (Vatican City State)
+VC Saint Vincent and the Grenadines
+VE Venezuela
+VG Virgin Islands, British
+VI Virgin Islands, U.S.
+VN Vietnam
+VU Vanuatu
+WF Wallis and Futuna
+WS Samoa
+YE Yemen
+YT Mayotte
+YU Serbia and Montenegro (Formally Yugoslavia)
+ZA South Africa
+ZM Zambia
+ZW Zimbabwe
 __END__
 
 =head1 NAME
 
-Geo::IPfree - Look up country of IP Address. This module make this off-line and the DB of IPs is free & small.
+Geo::IPfree - Look up the country of an IPv4 address
 
 =head1 SYNOPSIS
 
     use Geo::IPfree;
-    my ($country,$country_name) = Geo::IPfree::LookUp("192.168.0.1");
-
-    ... or ...
-
-    use Geo::IPfree qw(LookUp);
-    my ($country,$country_name) = LookUp("200.176.3.142");
-
-    ... or ...
-
-    use Geo::IPfree;
-    my $GeoIP = Geo::IPfree->new('/GeoIPfree/ipscountry.dat');
-    $GeoIP->Faster; ## Enable the faster option.
-    my ($country,$country_name,$ip) = $GeoIP->LookUp("www.cnn.com"); ## Getting by Hostname.
-
-    $GeoIP->LoadDB('/GeoIPfree/ips.dat');
-
-    my ($country,$country_name,$ip) = $GeoIP->LookUp("www.sf.net"); ## Getting by Hostname.
-
-    ... or ...
-
-    use Geo::IPfree;  
-    my $GeoIP = Geo::IPfree->new(); ## Using the default DB!
-    my ($country,$country_name) = $GeoIP->LookUp("64.236.24.28");
+    
+    my $geo = Geo::IPfree->new;
+    my( $code1, $name1 ) = $geo->LookUp( '200.176.3.142' );
+    
+    # use memory to speed things up
+    $geo->Faster;
+    
+    # lookup by hostname
+    my( $code2, $name2, $ip2 ) = $geo->LookUp( 'www.cnn.com' );
 
 =head1 DESCRIPTION
 
-This package comes with it's own database to look up the IP's country, and is totally free.
+Geo::IPfree is a Perl module that determines the originating country of an
+arbitrary IPv4 address. It uses a local file-based database to provide basic
+geolocation services.
+
+An updated version of the database can be obtained by visiting the Webnet77 
+website: L<http://software77.net/geo-ip/>.
   
 =head1 METHODS
 
-=head2 LoadDB
+=head2 new( [$db] )
+
+Creates a new Geo::IPfree instance. Optionally, a database filename may be
+passed in to load a custom data set rather than the version shipped with the
+module.
+
+=head2 LoadDB( $filename )
 
 Load the database to use to LookUp the IPs.
 
-=head2 LookUp
+=head2 LookUp( $ip|$hostname )
 
-Returns the ISO 3166 country (XX) code for an IP address or Hostname.
+Given an ip address or a hostname, this function returns three things:
 
-**If you send a Hostname you will need to be connected to the internet to resolve the host IP.
+=over 4
 
-=head2 Clean_Cache
+=item * The ISO 3166 country code (2 chars)
 
-Clean the memory used by the cache.
+=item * The country name
 
-=head2 Faster
+=item * The IP address resolved
+
+=back
+
+B<NB:> In order to use the location services on a hostname, you will need
+to have an internet connection to resolve a host to an IP address.
+
+=head2 Clean_Cache( )
+
+Clears any cached lookup data.
+
+=head2 Faster( )
 
 Make the LookUp() faster, good for big amount of LookUp()s. This will load all the DB in the memory (639Kb) and read from there,
 not from HD (good way for slow HD or network disks), but use more memory. The module "Memoize" will be enabled for some internal functions too.
 
 Note that if you make a big amount of querys to LookUp(), in the end the amount of memory can be big, than is better to use more memory from the begin and make all faster.
 
+=head2 nslookup( $host, [$last_lookup] )
+
+Attempts to resolve a hostname to an IP address. If it fails on the first pass
+it will attempt to resolve the same hostname with 'www.' prepended. C<$last_lookup>
+is used to supress this behavior.
+
+=head2 ip2nb( $ip )
+
+Encodes C<$ip> into a numerical representation.
+
+=head2 nb2ip( $number )
+
+Decodes C<$number> back to an IP address.
+
+=head2 dec2baseX( $number )
+
+Converts a base 10 (decimal) number to base 86.
+
+=head2 baseX2dec( $number )
+
+Converts a base 86 number to base 10 (decimal).
+
 =head1 VARS
 
 =over 4
@@ -456,6 +684,14 @@ The module looks for C<ipscountry.dat> in the following locations:
 
 =back
 
+=head1 SEE ALSO
+
+=over 4
+
+=item * http://software77.net/geo-ip/
+
+=back
+
 =head1 AUTHOR
 
 Graciliano M. P. E<lt>gm@virtuasites.com.brE<gt>
